/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS web site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* Standard includes. */
#include <stdio.h>

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

/* The task functions. */
static void prvDemoTask( void *pvParameters );
static void prvStatsTask( void *pvParameters );

/* The time base for the run time stats is generated by TMR01 (TMR0 and TMR1
combined).  TMR01 is configured to generate an interrupt each time the TMR01 
count value overflows.  By counting the overflows a 32-bit time value can
be constructed from the 16-bit timer.  The overflows are counted in the
prvTMR0OverflowIntHandler() ISR. */
static void prvTMR0OverflowIntHandler( void );

/* TMR0 is configured to generate an interrupt when it overflows.
ulTMR0OverflowCount is incremented in prvTRM0OverflowIntHandler() to count the
number of overflows that have occurred. */
volatile unsigned long ulTMR0OverflowCount = 0;

/*-----------------------------------------------------------*/

int main( void )
{
	/* Two tasks are created above the idle task priority (at tskIDLE_PRIORITY + 
	1) that just use up some processing time before blocking for a very short 
	period. */
	xTaskCreate( prvDemoTask, "Demo1", 240, NULL,	tskIDLE_PRIORITY + 1, NULL );
	xTaskCreate( prvDemoTask, "Demo2", 240, NULL, tskIDLE_PRIORITY + 1, NULL );

	/* The highest priority task (created at tskIDLE_PRIORITY + 2 ) runs every 
	five seconds and prints out the run time stats. */
	xTaskCreate( prvStatsTask, "Stats", 240, NULL, tskIDLE_PRIORITY + 2, NULL );

	/* Start the scheduler so the test task starts executing. */
	vTaskStartScheduler();	
	
	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

static void prvDemoTask( void *pvParameters )
{
volatile unsigned long ulLoopCounter;
const unsigned long ulMaxLoopCount = 0x1fffUL;
portTickType xLastExecutionTime;

/* The task will run every 5 milliseconds. */
const portTickType xBlockPeriod = ( 5 / portTICK_RATE_MS );

	/* Initialise xLastExecutionTime to the current time.  This is the only
	time this variable needs to be written to explicitly.  Afterwards it is
	updated internally within the xTaskDelayUnitl() API function. */
	xLastExecutionTime = xTaskGetTickCount();

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		/* Wait until it is time to run this task again. */
		vTaskDelayUntil( &xLastExecutionTime, xBlockPeriod );
		
		/* This loop is just to ensure the task uses up enough processing time
		to register in the run time statistics. */
		for( ulLoopCounter = 0; ulLoopCounter < ulMaxLoopCount; ulLoopCounter++ )
		{
			/* There is nothing to do here.  Just perform a "no operation" to
			ensure there are some instructions generated. */
			nop();
		}
	}
}
/*-----------------------------------------------------------*/

static void prvStatsTask( void *pvParameters )
{
portTickType xLastExecutionTime;

/* The buffer used to hold the run time stats text needs to be quite large.  It 
is therefore declared static to ensure it is not allocated on the task stack.  
This makes this function non re-entrant. */
static signed char cStringBuffer[ 512 ]; 

/* The task will run every 5 seconds. */
const portTickType xBlockPeriod = ( 5000 / portTICK_RATE_MS );

	/* Initialise xLastExecutionTime to the current time.  This is the only
	time this variable needs to be written to explicitly.  Afterwards it is
	updated internally within the xTaskDelayUntil() API function. */
	xLastExecutionTime = xTaskGetTickCount();

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		/* Wait until it is time to run this task again. */
		vTaskDelayUntil( &xLastExecutionTime, xBlockPeriod );

		/* Generate a text table from the run time stats.  This must fit into
		the cStringBuffer array. */
		vTaskGetRunTimeStats( cStringBuffer );
		
		/* Print out column headings for the run time stats table. */
		printf( "\nTask\t\tAbs\t\t%%\n" );
		printf( "----------------------------" );
		
		/* Print out the run time stats themselves. */
		printf( "%s", cStringBuffer );
	}
}
/*-----------------------------------------------------------*/

void vSetupTimerForRunTimeStats( void )
{
	/* This function requires knowledge of the individual bit fields of the 
	RX600 TMR0 and TMR1 registers.  The bits within these registers are
	described in the RX600 hardware manual, which can be obtained from the 
	Renesas	web site. */

	/* Enable TMR0. */
	MSTP( TMR0 ) = 0;

	/* Configure TMR0 to generate overflow interrupts. */
	TMR0.TCR.BIT.OVIE = 1;
	
	/* TMR0 counts TMR1 overflows to create a 16bit timer from the two 8 bit
	counters. */
	TMR0.TCCR.BIT.CSS = 3;
	
	/* TMR1 counts from the 48MHz PCLK / 1024. */
	TMR1.TCCR.BIT.CSS = 1;
	TMR1.TCCR.BIT.CKS = 5;
	
	/* Enable the overflow interrupt... */
	_IEN( _TMR0_OVI0 ) = 1;

	/* ...and set its priority to the maximum possible, this is above the priority
	set by configMAX_SYSCALL_INTERRUPT_PRIORITY so will nest with all other
	interrupts - including those generated by the kernel. */
	_IPR( _TMR0_OVI0 ) = 15;
}
/*-----------------------------------------------------------*/

#pragma interrupt ( prvTMR0OverflowIntHandler( vect = _VECT( _TMR0_OVI0 ), enable ) )
static void prvTMR0OverflowIntHandler( void )
{
	/* Just count the number of interrupts.  This count is shifted to the two most
	significant bytes and combined with the current TMR01 counter value to generate
	a 32 bit run time counter value from a 16 bit counter. */
	ulTMR0OverflowCount++;
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}
/*-----------------------------------------------------------*/

signed char *sbrk( size_t size )
{
	/* Just included to keep the linker quiet.  This function is not actually
	called anywhere. */
	return NULL;
}


